#include <stdio.h>
#include <cv.h>
#include <highgui.h>
 
void DrawNextContour(IplImage *img, CvSeq *Contour, int Level);
void DrawChildContour(IplImage *img, CvSeq *Contour, int Level);
void GetContourFeature(CvSeq *Contour, double *Area, int *maxx, int *maxy, int *minx, int *miny);
int AreaN=0;

int main (int argc, char *argv[]) {
  CvPoint offset=cvPoint(0,0);
    IplImage *Image,*img_gray;
    CvSeq *Contour;
    CvMemStorage *Storage=cvCreateMemStorage (0);
    CvScalar ContoursColor;
    ContoursColor=CV_RGB( 255, 0, 0 );
    int fco = 0;

    cvNamedWindow("Contours",CV_WINDOW_AUTOSIZE);
  if (argc < 2) {
    printf("Usage: edgecapture imagename\n");
    return -1;
  }
  if((Image = cvLoadImage(argv[1], CV_LOAD_IMAGE_COLOR)) == 0) {
    printf("Error: Failed to load %s\n", argv[1]);
    return -1;
  }

    img_gray=cvCreateImage(cvGetSize(Image),IPL_DEPTH_8U,1);
    if(Image->nChannels > 1)
       cvCvtColor(Image,img_gray,CV_BGR2GRAY);
    else
       cvCopy(Image, img_gray, 0);

    cvThreshold(img_gray,img_gray,100,255,CV_THRESH_BINARY);
    cvFindContours(img_gray,Storage,&Contour,sizeof(CvContour),CV_RETR_TREE,CV_CHAIN_APPROX_NONE, offset);

      if (Contour != NULL){
        DrawNextContour(Image, Contour, 1);
      } else {
        printf("contour is null\n");
      }

    while(1)
    {
        cvShowImage("Contours",Image);
        cvMoveWindow("Contours",0,0);

        if(cvWaitKey(1)>=0) {
            break;
        } else if(fco == 250) {
            break;
        } else {
            fco++;
        }
    }

    cvDestroyWindow("Contours");
    cvReleaseImage(&Image);
    cvReleaseMemStorage (&Storage);
 
    return 0;
}

void DrawNextContour(
                IplImage *img,    
                CvSeq *Contour, 
                int Level            
                ){

  int line_type=8;
  CvPoint offset=cvPoint(0,0);
  double area;
  int maxx, maxy, minx, miny;
  char text[20];
 CvFont font;
    char savefile[20];

        cvInitFont(&font, CV_FONT_HERSHEY_SIMPLEX, 0.5, 0.5, 0, 2, 4);

      CvScalar ContoursColor;

      if ((Level % 2) == 1){
            ContoursColor = CV_RGB( 255, 0, 0 );
      }else{
            ContoursColor = CV_RGB( 0, 0, 255 );
      }

      maxx = 0; maxy = 0; minx = img->width; miny = img->height;
      GetContourFeature(Contour, &area, &maxx, &maxy, &minx, &miny);
      sprintf(text, "%10.0f", area);
      if(area > 25000) {
         cvDrawContours( img, Contour, ContoursColor, ContoursColor, 0, 2, line_type, offset);

         cvPutText(img, text, cvPoint (minx+5, miny+20), &font, ContoursColor);

         cvSetImageROI(img, cvRect(minx,miny,maxx-minx,maxy-miny));
         sprintf( savefile, "AREA%d.jpg", AreaN);
         printf( "%s\n", savefile );
         cvSaveImage(savefile, img, 0);
         cvResetImageROI(img);
        cvLine (img, cvPoint (maxx, maxy), cvPoint (maxx, miny), ContoursColor, 3, 8, 0);
        cvLine (img, cvPoint (minx, maxy), cvPoint (minx, miny), ContoursColor, 3, 8, 0);
        cvLine (img, cvPoint (maxx, maxy), cvPoint (minx, maxy), ContoursColor, 3, 8, 0);
        cvLine (img, cvPoint (maxx, miny), cvPoint (minx, miny), ContoursColor, 3, 8, 0);

         AreaN++;
      }
      if (Contour->h_next != NULL)
            DrawNextContour(img, Contour->h_next, Level);

      if (Contour->v_next != NULL)
            DrawChildContour(img, Contour->v_next, Level + 1);
}

void DrawChildContour(
                IplImage *img,    
                CvSeq *Contour, 
                int Level             
                ){

  int line_type=8;
  CvPoint offset=cvPoint(0,0);
  char text[20];
  double area;
  int maxx, maxy, minx, miny;
 CvFont font;
    char savefile[20];

        cvInitFont(&font, CV_FONT_HERSHEY_SIMPLEX, 0.5, 0.5, 0, 2, 4);

      CvScalar ContoursColor;

      if ((Level % 2) == 1){
            ContoursColor = CV_RGB( 255, 0, 0 );
      }else{
            ContoursColor = CV_RGB( 0, 0, 255 );
      }

      maxx = 0; maxy = 0; minx = img->width; miny = img->height;
      GetContourFeature(Contour, &area, &maxx, &maxy, &minx, &miny);
      sprintf(text, "%10.0f", area);
      if(area > 25000) {
         cvDrawContours( img, Contour, ContoursColor, ContoursColor, 0, 2, line_type, offset);

         cvPutText(img, text, cvPoint (minx+5, miny+20), &font, ContoursColor);

         cvSetImageROI(img, cvRect(minx,miny,maxx-minx,maxy-miny));
         sprintf( savefile, "AREA%d.jpg", AreaN);
         printf( "%s\n", savefile );
         cvSaveImage(savefile, img, 0);
         cvResetImageROI(img);
        cvLine (img, cvPoint (maxx, maxy), cvPoint (maxx, miny), ContoursColor, 3, 8, 0);
        cvLine (img, cvPoint (minx, maxy), cvPoint (minx, miny), ContoursColor, 3, 8, 0);
        cvLine (img, cvPoint (maxx, maxy), cvPoint (minx, maxy), ContoursColor, 3, 8, 0);
        cvLine (img, cvPoint (maxx, miny), cvPoint (minx, miny), ContoursColor, 3, 8, 0);

         AreaN++;
      }

      if (Contour->h_next != NULL)
            DrawNextContour(img, Contour->h_next, Level);
      if (Contour->v_next != NULL)
            DrawChildContour(img, Contour->v_next, Level + 1);
}

void GetContourFeature(CvSeq *Contour, double *Area, int *maxx, int *maxy, int *minx, int *miny){
  int i;
         Area[0] = fabs(cvContourArea(Contour, CV_WHOLE_SEQ, 0));
         double Perimeter = cvArcLength(Contour, CV_WHOLE_SEQ, -1);
         double CircleLevel = 4.0 * CV_PI * Area[0] / (Perimeter * Perimeter);


         CvRect rect = cvBoundingRect(Contour, 0);

           for ( i = 0; i < Contour->total; i++){
              CvPoint *point = CV_GET_SEQ_ELEM (CvPoint, Contour, i);
             if(point[0].x > maxx[0]) maxx[0] = point[0].x;
             if(point[0].y > maxy[0]) maxy[0] = point[0].y;
             if(point[0].x < minx[0]) minx[0] = point[0].x;
             if(point[0].y < miny[0]) miny[0] = point[0].y;
           }
}

